/*
 * Copyright (C) 2012, Eric Wong <normalperson@yhbt.net>
 * License: GPLv3 or later (see COPYING for details)
 */
#include "cmogstored.h"
#include "digmd5.h"

struct md5_ctx * mog_md5_new(void)
{
	struct md5_ctx *ctx = mog_cachealign(sizeof(struct md5_ctx));

	md5_init_ctx(ctx);

	return ctx;
}

ssize_t mog_md5_read(struct md5_ctx *ctx, int fd)
{
	size_t len;
	void *buf = mog_fsbuf_get(&len);
	ssize_t r;

retry:
	r = read(fd, buf, len);
	if (r > 0) { /* most likely */
		md5_process_bytes(buf, r, ctx);
	} else if (r == 0) {
		/* wait for user to call mog_md5_hex() */
	} else {
		assert(r < 0 && errno && "buggy read(2)?");
		if (errno == EINTR) goto retry; /* may happen on crazy FSes */
		/* bail on EAGAIN, too, not possible on regular files */
	}

	return r;
}

void mog_md5_hex(struct md5_ctx *ctx, char *buf, size_t len)
{
	static const char hex[] = "0123456789abcdef";
	unsigned char result[16];
	unsigned char *b = result;
	char *out = buf;
	int i;

	md5_finish_ctx(ctx, result);

	assert(len >= (sizeof(result) * 2) && "hex buffer too small");

	for (i = 0; i < sizeof(result); i++) {
		*out++ = hex[*b >> 4];
		*out++ = hex[*b & 0x0f];
		b++;
	}
}
